Puppet::Type.type(:ipsec_secrets).provide :libreswan do
  @doc = %q{ Manages ipsec.secrets using libreswan. }

  commands :ipsec => 'ipsec'

  def self.instances
    ipsec_secrets = []
    hash = {}
    Dir['/etc/ipsec.d/*.secrets'].each do |file_name|
      File.open(file_name, 'r') do |file|
        debug 'File: %s' % file_name
        file.read.each_line do |line|
          debug 'Line: %s' % line.strip
          if line =~ /([\w\.\@:-]+)\s+([\w\.\@:-]+)\s*:\s*PSK\s+\"(.+)\"/
            leftid = $1
            rightid = $2
            psk = $3
            hash = {}
            hash[:ensure] = :present
            hash[:name] = "#{leftid} #{rightid}"
            hash[:psk] = psk
            hash[:file_name] = File.basename(file_name)

            debug 'Proprty_hash: %s' % hash

            found_duplicates = false
            # Remove duplicates.
            ipsec_secrets.each do |ipsec_secret|
              if ipsec_secret.name == hash[:name]
                ipsec_secret.destroy
                found_duplicates = true
                break
              end
            end

            unless found_duplicates
              ipsec_secrets << new(hash)
              debug 'Instance: %s' % [ hash[:name], hash[:psk] ].join(' ')
            end
          end
        end
      end
    end

    # Check file name.
    ipsec_secrets.each do |ipsec_secret|
      debug 'Methods: %s' % ipsec_secret.inspect.to_s
      file_name_should_be = [
          ipsec_secret.name.downcase.gsub(/[\s\.\@:-]/,'_'),
          'secrets'
        ].join('.')
      debug 'Validate: %s' % [ ipsec_secret.name, ipsec_secret.psk,
        ipsec_secret.file_name ].join(' ')
      # Change file name.
      unless ipsec_secret.file_name == file_name_should_be
        ipsec_secret.file_name = file_name_should_be
      end
      ipsec_secrets.validate
    end

    ipsec_secrets
  end

  def self.prefetch(resources)
    debug '[prefetch]'
    ipsec_secrets = instances
    found_providers = []
    resources.keys.each do |name|
      if provider = ipsec_secrets.find { |ipsec_secret| ipsec_secret.name == name }
        resources[name].provider = provider
        found_providers << provider
      end
    end
    # Destroy all unused secrets file
    (ipsec_secrets - found_providers).each { |provider| provider.destroy }
  end

  def create
    file_name = [ @resource[:name].gsub(/[\s\.\@:-]/, '_'), 'secrets' ].join('.')
    debug 'Create: %s.' % file_name
    (leftid, rightid) = @resource[:name].split(/\s/)
    psk = @resource[:psk]
    File.new("/etc/ipsec.d/#{file_name}") do |f|
      f.write("#{leftid} #{rightid} : PSK \"#{psk}\"")
    end
    @property_hash[:ensure] = :present
    refresh
  end

  def destroy
    name = @property_hash[:name]
    file_name = @property_hash[:file_name]
    debug 'Destroy: %s.' % [ name, file_name ].join(' with file: ')
    File.delete("/etc/ipsec.d/#{file_name}")
    refresh
    # @property_hash.clear
  end

  def exists?
    @property_hash[:ensure] == :present
  end

  def file_name
    @property_hash[:file_name] || :absent
  end

  def file_name=(value)
    if File.exist?("/etc/ipsec.d/#{@property_hash[:file_name]}")
      debug 'Rename: %s.' % [ @property_hash[:file_name], value ].join(' => ')
      File.rename("/etc/ipsec.d/#{@property_hash[:file_name]}",
        "/etc/ipsec.d/#{value}")
      @property_hash[:file_name] = value
    else
      @property_hash[:file_name] = value
      create
    end
  end

  def psk
    @property_hash[:psk] || :absent
  end

  def psk=(value)
    create
  end

  def refresh
    debug 'IPsec secrets refresh.'
    ipsec('secrets')
  end

  def validate
    file_name = @property_hash[:file_name]
    debug 'Validate: %s' % file_name
    unless File.exist?("/etc/ipsec.d/#{file_name}")
      create
    end
  end
end
