Puppet::Type.newtype(:ipsec_secrets) do
  @doc = %q{
    This type provides the capability to manage ipsec.secrets files
    with puppet.

    Example:

      ipsec_secrets { 'leftid rightid':
        ensure => present,
        psk     => 'preshared secret key',
      }
  }

  ensurable

  newparam(:name) do
    desc %q{ The pair contains left and right id. }
    newvalues(/\A([\w\.\@\%:-]+)\s([\w\.\@\%:-]+)\Z/)
  end

  newproperty(:psk) do
    desc %{ Preshared secret key. }
  end

  newparam(:file_name) do
    desc %q{ Readonly. File name. }
  end

  autorequire(:package) do
    case value(:provider)
    when :libreswan
      %w{libreswan}
    else
      []
    end
  end

  autorequire(:service) do
    case value(:provider)
    when :libreswan
      %w{ipsec}
    else
      []
    end
  end
end
